package com.swago.room.widget

import android.content.Context
import android.util.AttributeSet
import android.util.Log
import android.util.TimeUtils
import android.view.LayoutInflater
import android.view.View
import androidx.constraintlayout.widget.ConstraintLayout
import androidx.recyclerview.widget.LinearLayoutManager
import androidx.recyclerview.widget.RecyclerView
import com.alibaba.android.arouter.launcher.ARouter
import com.bumptech.glide.Glide
import com.swago.baseswago.R
import com.swago.baseswago.baseroom.*
import com.swago.baseswago.constant.ARouteConstant
import com.swago.baseswago.model.live.RoomUserModel
import com.swago.baseswago.util.AppContext
import com.swago.baseswago.util.NoDoubleClickListener
import com.swago.baseswago.util.TimeUtil
import com.swago.room.adapter.RoomUserAdapter
import com.swago.room.databinding.LayoutUserHeaderViewBinding
import com.swago.room.dialog.AudioNoticeDialog
import com.swago.room.dialog.AudioNoticeUserDialog
import com.swago.room.inter.IHeader

/**
 *@date 2022/3/4 14:56
 *description:
 */
class ComHeaderView  : ConstraintLayout, IRoomActiveListener, IHeader, RoomTimer.TimeTickListener {


    lateinit var binding: LayoutUserHeaderViewBinding

    var followFun:((id:String)->Unit)? = null
    var showUserInfo:((id:String)->Unit)? = null
    var openAudienceListDialog:(()->Unit)? = null
    var showAudioUserNoticeFun:(()->Unit)? = null

    //钻石收入
    private var localIncome = 0L

    private var timeL = 0L

    private val adapter by lazy {
        RoomUserAdapter()
    }

    constructor(context: Context) : this(context, null)
    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)
    constructor(context: Context, attrs: AttributeSet?, defStyle: Int) : super(
        context,
        attrs,
        defStyle
    ) {
        initView(context)
    }

    private fun initView(context: Context) {
        RoomTimer.timeTickList.add(this)
        SwagoRoomManager.addListener(this)
        binding = LayoutUserHeaderViewBinding.inflate(LayoutInflater.from(context), this, true)
        binding.rvUser.layoutManager = LinearLayoutManager(context,RecyclerView.HORIZONTAL,false)
        binding.rvUser.adapter = adapter

        binding.tvFollow.setOnClickListener(object : NoDoubleClickListener() {
            override fun onClick() {
                SwagoRoomManager.iRoomInfo?.let {
                    followFun?.invoke(it.getAnchorId())
                }
            }
        })

        binding.tvIncome.setOnClickListener(object:NoDoubleClickListener(){
            override fun onClick() {
                ARouter.getInstance()
                    .build(ARouteConstant.Room.rankList)
                    .withString("roomId",SwagoRoomManager.iRoomInfo?.getRoomId()?:"")
                    .navigation()
            }
        })

        binding.cl.setOnClickListener(object:NoDoubleClickListener(){
            override fun onClick() {
                SwagoRoomManager.iRoomInfo?.let {
                    showUserInfo?.invoke(it.getAnchorId())
                }
            }
        })

        adapter.setOnItemClickListener { _, _, position ->
            if (adapter.data.size>position){
                showUserInfo?.invoke(adapter.data[position].user_id)
            }
        }

        binding.tvNum.setOnClickListener(object:NoDoubleClickListener(){
            override fun onClick() {
                openAudienceListDialog?.invoke()
            }
        })

        binding.tvAnnouncement.setOnClickListener(object:NoDoubleClickListener(){
            override fun onClick() {
                if (SwagoRoomManager.roleType == RoleType.user){
                    showAudioUserNoticeFun?.invoke()
                }
            }
        })
    }


    fun updateIncome(income:String){
        if (income.toLong()>localIncome){
            localIncome = income.toLong()
            binding.tvIncome.text = income
        }
    }

    fun updateHotValue(hotValue:String){
        binding.tvHot.text = hotValue
    }

    override fun changeRoom(iRoomInfo: IRoomInfo) {
        binding.clRoot.visibility = View.INVISIBLE
    }

    override fun leaveRoom(iRoomInfo: IRoomInfo) {
        adapter.data.clear()
        adapter.notifyDataSetChanged()
    }

    override fun joinedRoom(iRoomInfo: IRoomInfo) {
        binding.clRoot.visibility = View.VISIBLE
        Glide.with(context).load(iRoomInfo.getRoomCover())
            .placeholder(R.mipmap.default_avatar)
            .error(R.mipmap.default_avatar).into(binding.ivAvatar)
        binding.tvName.text =iRoomInfo.getAnchorName()
        binding.tvId.text = AppContext.getContext().resources.getString(R.string.id_x).format(iRoomInfo.getAnchorCode())
        binding.tvHot.text = iRoomInfo.getRoomHot()
        binding.tvIncome.text = iRoomInfo.getRoomIncome()
        localIncome = iRoomInfo.getRoomIncome().toLong()

        if (SwagoRoomManager.roleType == RoleType.anchor){
            binding.tvFollow.visibility = View.GONE
            binding.tvTime.visibility = View.VISIBLE
        }else{
            if (iRoomInfo.getIsFollow()==1){
                binding.tvFollow.visibility = View.GONE
            }else{
                binding.tvFollow.visibility = View.VISIBLE
            }
        }
        if (iRoomInfo.getRoomType()==2 && iRoomInfo.getAudioNotice().isNotEmpty()){
            binding.tvAnnouncement.visibility = View.VISIBLE
            binding.tvAnnouncement.text = iRoomInfo.getAudioNotice()
        }else{
            binding.tvAnnouncement.visibility = View.GONE
        }
    }

    override fun endRoom(iRoomInfo: IRoomInfo?) {
    }

    override fun closeRoomed() {
        RoomTimer.timeTickList.remove(this)
        SwagoRoomManager.removeListener(this)
    }

    override fun getHeaderView(): View {
        return this
    }

    fun updateRoomUser(data: List<RoomUserModel.ListBean>,count:Int) {
        adapter.setNewData(data)
        if (data.isNotEmpty()){
            binding.rvUser.smoothScrollToPosition(0)
            if (count==0){
                binding.tvNum.visibility = View.INVISIBLE
            }else{
                binding.tvNum.text = if (count>99) "99+" else "$count"
                binding.tvNum.visibility = View.VISIBLE
            }
        }
    }

    fun setFollow(){
        binding.tvFollow.visibility = View.GONE
    }

    override fun onTimeCount(time: Int) {
        if(SwagoRoomManager.roleType == RoleType.anchor){
            timeL ++
            binding.tvTime.text = TimeUtil.formatMin(timeL)
        }
    }

    fun updateAudioNotice(notice:String){
        if(notice.isEmpty()){
            binding.tvAnnouncement.visibility = View.GONE
            binding.tvAnnouncement.text = ""
        }else{
            binding.tvAnnouncement.visibility = View.VISIBLE
            binding.tvAnnouncement.text = notice
        }
    }
}